# This file is generated by pto-pre-build.sh. Do not edit it manually.
# Mon 24 Jun 2024 04:06:51 AM EDT
# SOURCE NAME: jgtml_cli-ttf-proto-01-2406181221.py
# ------------------------------------
import argparse
import pandas as pd
#from jgtpy import JGTCDSSvc as svc
#from jgtutils import jgtpov as jpov
#from jgtutils.jgtconstants import (MFI_VAL, ZCOL, AO)

from jgtml.ptottf import create_ttf_csv

import os

def main():
  parser = argparse.ArgumentParser(description="Create ttf CSV file")
  parser.add_argument("-i", "--instrument", required=True, help="Instrument name")
  parser.add_argument("-t", "--timeframe", required=True, help="Timeframe (e.g., D1, H4)")
  parser.add_argument("-uf", "--full", action="store_true", help="Use full dataset")
  parser.add_argument("-new", "--fresh", action="store_true", help="Use fresh data")
  parser.add_argument("-fr", "--force_read", action="store_true", help="Force to read CDS (should increase speed but relies on existing data)")
  parser.add_argument("-c", "--quotescount", type=int, default=-1, help="Number of quotes to retrieve (default: 333)")

  args = parser.parse_args()
  create_ttf_csv(args.instrument, args.timeframe, args.full if args.full else False, args.fresh, args.quotescount, args.force_read)

if __name__ == "__main__":
  main()
  